/****************************************************************************
 *
 *  Copyright (c) 2012, EBM WebSourcing
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

package org.ow2.petals.admin.jmx;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.artifact.Logger;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.InvalidLogLevelException;
import org.ow2.petals.admin.api.exception.LoggerNotFoundException;
import org.ow2.petals.admin.jmx.topology.TopologyUtils;
import org.ow2.petals.admin.topology.Domain;
import org.ow2.petals.jmx.JMXClient;
import org.ow2.petals.jmx.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.exception.ConnectionErrorException;
import org.ow2.petals.jmx.exception.LoggerDoesNotExistException;
import org.ow2.petals.jmx.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.exception.PetalsAdminServiceErrorException;

/**
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class JMXContainerAdministration implements ContainerAdministration {

    protected JMXContainerAdministration() {
    }

    @Override
    public void connect(String host, int port, String user, String password)
            throws ContainerAdministrationException {
        try {
            JMXClientConnection.createJMXClient(host, port, user, password);
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        }
    }

    @Override
    public void disconnect() throws ContainerAdministrationException {
        try {
            JMXClientConnection.disconnectJMXClient();
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        }
    }

    @Override
    public String getSystemInfo() throws ContainerAdministrationException {
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            String systemInfo = client.getAdminServiceClient().getSystemInfo();
            return systemInfo;
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        } catch (AdminServiceErrorException asee) {
            throw new ContainerAdministrationException(asee);
        } catch (AdminDoesNotExistException adnee) {
            throw new ContainerAdministrationException(adnee);
        }
    }

    @Override
    public void stopContainer() throws ContainerAdministrationException {
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            client.getPetalsAdminServiceClient().stopContainer();
            JMXClientConnection.disconnectJMXClient();
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        } catch (PetalsAdminServiceErrorException pasee) {
            throw new ContainerAdministrationException(pasee);
        } catch (PetalsAdminDoesNotExistException padnee) {
            throw new ContainerAdministrationException(padnee);
        }
    }

    @Override
    public void shutdownContainer() throws ContainerAdministrationException {
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            client.getPetalsAdminServiceClient().shutdownContainer();
            JMXClientConnection.disconnectJMXClient();
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        } catch (PetalsAdminServiceErrorException pasee) {
            throw new ContainerAdministrationException(pasee);
        } catch (PetalsAdminDoesNotExistException padnee) {
            throw new ContainerAdministrationException(padnee);
        }
    }

    @Override
    public Domain getTopology() throws ContainerAdministrationException {
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            Set<Map<String, String>> topology = client.getPetalsAdminServiceClient().retrieveTopology();
            return TopologyUtils.createListOfDomains(topology);
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        } catch (PetalsAdminServiceErrorException pasee) {
            throw new ContainerAdministrationException(pasee);
        } catch (PetalsAdminDoesNotExistException padnee) {
            throw new ContainerAdministrationException(padnee);
        }
    }

    @Override
    public Properties getServerProperties() throws ContainerAdministrationException {
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            return client.getPetalsAdminServiceClient().retrieveServerProperties();
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        } catch (PetalsAdminServiceErrorException pasee) {
            throw new ContainerAdministrationException(pasee);
        } catch (PetalsAdminDoesNotExistException padnee) {
            throw new ContainerAdministrationException(padnee);
        }
    }

    @Override
    public void changeLoggerLevel(String loggerName, String loggerLevel)
            throws LoggerNotFoundException, InvalidLogLevelException,
            ContainerAdministrationException {
        // TODO: LoggerNotFoundException and InvalidLogLevelException must be
        // thrown here according to the result of setLevelForLogger because the
        // container knows its log level and loggers.
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            client.getLoggerServiceClient().setLevelForLogger(loggerName, loggerLevel);
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        } catch (LoggerServiceErrorException lsee) {
            throw new ContainerAdministrationException(lsee);
        } catch (LoggerDoesNotExistException ldnee) {
            throw new ContainerAdministrationException(ldnee);
        }
    }
    
    @Override
    public List<Logger> getLoggers()
            throws ContainerAdministrationException {
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            final String[][] loggersArray = client.getLoggerServiceClient().getAllLoggers();
            final List<Logger> loggers = new LinkedList<Logger>();
            for (int i = 0; i < loggersArray.length; i++) {
                loggers.add(new Logger(loggersArray[i][0], loggersArray[i][1]));
            }
            return loggers;
        } catch (ConnectionErrorException cee) {
            throw new ContainerAdministrationException(cee);
        } catch (LoggerServiceErrorException lsee) {
            throw new ContainerAdministrationException(lsee);
        } catch (LoggerDoesNotExistException ldnee) {
            throw new ContainerAdministrationException(ldnee);
        }
    }
}
